// Output generated from file: //production/Public/Fractals/Fragmentarium/Frags/Dev_3_Matchmoving.frag
// Created: Pn lip 27 02:42:08 2015
// Based on original classic Quaternion bundled with Fragmentarium
// Modifications added by Patryk Kizny / kizny.com

// 2015/06/15
// - Added Box folding
// - Added Sphere Folding
// - Added Volume clipping
// - Added general fractal repositioning

#info 4D Quaternion Julia DE + Box Folding + Sphere Folding

// Flags
#define providesInit
#define USE_EIFFIE_SHADOW
#define PERFECT_DE
#define USE_EXTERNAL_CAMERA // Todo
#define USE IBL // Todo
#define providesBackground
//#define transparentBackground

// Uncomment this to output alpha pass
#define TAG_LAYERS
// Uncomment this to output depth pass
#define DEPTH_PASS

//#define USE_IQ_CLOUDS
//#define USE_IQ_CLOUDS_MULTILAYER // Multilayer Clouds by PK
//#define KN_VOLUMETRIC

uniform float time;

// Core

#vertex
#include "CameraData_2.frag"
#endvertex

#include "Presets-Camera-Z-Up.frag"
#include "MathUtils.frag"
#include "Quaternion-Maths.frag"
#include "SpaceTransforms.frag"
#include "DE-PK-1.0.9.frag"

#group 4D Quaternion Julia
// Number of fractal iterations.
uniform int QTIterations;  slider[0,16,100]

// Breakout Distance
uniform float QTThreshold; slider[0,10,100]

// Quaterion Constant C
uniform vec4 QTC; slider[(-1,-1,-1,-1),(0.18,0.88,0.24,0.16),(1,1,1,1)]

// Iterative Rotation
uniform vec3 rotVector; slider[(0,0,0),(0,0,1),(1,1,1)]
uniform float rotAngle; slider[0.00,0,180]

// Iterative Scaling
uniform float Scale;  slider[-3.0,1,3.0]

#group box Folding
// Added box folding by Patryk Kizny
// Sphere folding does not seem to work for this fractal

uniform int foldBoxStartIt; slider[0,0,256]
uniform int foldBoxIts; slider[0,0,16]

uniform vec3 foldBoxLimit; slider[(-5,-5,-5),(0,0,0),(5,5,5)]
uniform vec3 foldBoxValue; slider[(-5,-5,-5),(2,2,2),(5,5,5)]
uniform vec3 foldBoxOffset; slider[(-5,-5,-5),(0.,0.,0.),(5,5,5)]

#group sphere Folding
uniform int foldSphereStartIt; slider[0,0,256]
uniform int foldSphereIts; slider[0,0,16]
uniform float foldSphereMinRad;  slider[0,0.25,2.0]

#group Post
uniform sampler2D Background; file[Ditch-River_2k.hdr]
uniform vec2 bgScale; slider[(0,0),(1,1),(1,1)]
uniform vec2 bgOffset; slider[(0,0),(1,1),(1,1)]

// ---------------------------------------- Init -------------------------------------------

mat3 rot; // QT
void init() {
	rot = rotationMatrix3(normalize(rotVector),rotAngle);
}

// ---------------------------------------- Quaternion Modified -------------------------------------------

vec4 scale = vec4(Scale, Scale, Scale, abs(Scale)) / foldSphereMinRad;

// DE implementation here based on Syntopia + Subblue + Keenan Crane
// http://www.subblue.com/blog/2009/9/20/quaternion_julia
// Added box folding, sphere folding and iterative rotation

float DEQT(vec3 pos) {
	
	vec4 dp = vec4(1.0, 0.0,0.0,0.0);
	vec4 p = vec4(pos, 0.0);

	for (int i = 0; i <QTIterations; i++){
		
		// Quaternion interations
		// Syntopia's compressed version
		//	dp = 2.0* vec4(p.x*dp.x-dot(p.yzw, dp.yzw), p.x*dp.yzw+dp.x*p.yzw+cross(p.yzw, dp.yzw));
		//	p = vec4(p.x*p.x-dot(p.yzw, p.yzw), vec3(2.0*p.x*p.yzw)) + C;
		
		// Experimental, generates artifacts
		//	dp = QTDerCons * qMult(p,dp);
		dp = 2.0 * qMult(p,dp);
		p = qSq(p) + QTC;
		
		// rotation
		p.xyz *= rot;
		
		// scaling, experimental, generates artifacts
		// p *= scale;
		
		// Box Folding (v2) - separate axes + value + offset
		if (foldBoxIts > 0){
			if ( i >= foldBoxStartIt && i < foldBoxStartIt + foldBoxIts){
				p.xyz = clamp(p.xyz, -foldBoxLimit, foldBoxLimit) * foldBoxValue - p.xyz + foldBoxOffset;
			}
		}
		
		// Sphere folding
		if(foldSphereIts > 0){
			if ( i >= foldSphereStartIt && i < foldSphereStartIt + foldSphereIts){
				float r2 = dot(p.xyz, p.xyz);
				p *= clamp(max(foldSphereMinRad/r2, foldSphereMinRad), 0.0, 1.0);
			}
		}
		
		float p2 = dot(p,p);
		orbitTrap = min(orbitTrap, abs(vec4(p.xyz,p2)));
		if (p2 > QTThreshold) break;
	}
	
	// DE
	float r = length(p);
	return  0.5 * r * log(r) / length(dp);
}


// ----------------------------------------Main -------------------------------------------

float DE(vec3 z) {

	// get slice volume
	vec3 z1;
	z1 = opRotate(z, cBoxRotate, length(cBoxRotate));
	z1 = opTranslate(z1, cBoxTranslate - sTranslate);
	float deBox = DEBox(z1, cBoxSize);
	
	float deBox2 = DEBox(vec3(z.x+0.25,z.y-0.5,z.z), vec3(2.0,2.0,0.001));


	// apply global transformations
	z = opNUScale(z, sScale);
	z = opTranslate(z, sTranslate);
	z = opRotate(z, normalize(sRotate), length(sRotate));
//	z = opTwist(z,sTwist);

	// Stereo projection
	z = stereoMap(z);
	
	// Noise
	z = spaceNoise(z);

	//  Instantiate
	//	z.xz = mod(z.xz,5) -0.0; //+vec2(0.1); //spacingXY
	
	//get fractal
	float deQT = DEQT(z);
 
	// return plain fractal
	float DE = deQT;
	
	// clip to volume
	if (clipToBox){
		DE = max(deQT, deBox);
	}
	// disply cutting volume
	if (showClippingBox) {
		DE = min(deQT, deBox);
	}
//	DE = min(DE, deBox2);
	return DE;

}

// Returns flat background texture
vec3 backgroundColor(vec3 dir, vec2 pos){
	//	sampler2D Background = file("//production/Public/Fractals/Fragmentarium/Camera Test/Frames/Test3/Test_00000.jpg");
		pos = pos * bgScale + bgOffset; 
		return texture2D(Background,pos).xyz;
}

// ---------------------------------------- Presets -------------------------------------------







#preset default
sTranslate = 0.2686,-0.5672,0
sRotate = 0,0,0
sScale = 1,1,1
sTwist = 0
sStereo = false
sStereoF = 2
sStereoD1 = 1
sStereoD2 = 2
sStereoOffset = 0,0,0
sStereoMix = 1
clipToBox = true Locked
showClippingBox = false Locked
cBoxSize = 1.5,1.5,3
cBoxTranslate = 0,0,3
cBoxRotate = 0,0,0
nTime = 1
nScale = 1,1,1
nAmplitude = 0,0,0
FOV = 1.35
UseTrueFOV = true
TrueFOV = 75
UseFocalLength = false
FocalLength = 25.6745
FilmGate = 20.25
Eye = -0.0321645,4.56732,0.546415
Ey_ = 0,0,0
Target = -0.316827,1.58125,0.497566
Targe_ = 0,0,0
iCamWeight = 0
iCamScale = 2
iCamPosMod = 1,1,1
iCamDirMod = 1,1,-1
iCamPosOffs = 0,0,0
iCamDirOffs = 0,0,0
iCamFrame = 8.975
FocalPlane = 1
Aperture = 0
InFocusAWidth = 0
ApertureNbrSides = 2
ApertureRot = 0
ApStarShaped = false
Gamma = 2
ToneMapping = 3
Exposure = 1
Brightness = 1
Contrast = 1
Saturation = 1
GaussianWeight = 1
AntiAliasScale = 2
Bloom = false
BloomIntensity = 0.25
BloomPow = 2
BloomTaps = 4
Detail = -3.6
DetailAO = -0.5
AutoFocus = false
FudgeFactor = 0.95
MaxRaySteps = 768
MaxDistance = 32
MaxRayStepsDiv = 2
Dither = 0.5
NormalBackStep = 1
DepthScale = 1
DepthOffset = 0
DepthOffsetPost = 0
DepthScalePower = 0
MultiChannelDepth = false
FlatPass = false
OutputDepth = false
tagBackground = 0,0,0
tagLight = 1,0,0
tagFloor = 0,1,0
tagObject = 0,1,1
tagFractal = 1,1,1
OutputTag = false
AO = 0,0,0,0.5
AoCorrect = 0
Specular = 0.01
SpecularExp = 64
SunLight = 1,1,1,0.3
SunDir = 0,0.1
QualityShadows = true
DebugSun = false
AmbientLight = 1,1,1,0
AmbientLightMin = 0
Glow = 1,1,1,0
GlowMax = 256
Fog = 1
FogExp = 2
Reflection = 0.0784314,0.0784314,0.0784314
ReflectionsNumber = 0
SpotGlow = true
SpotLight = 1,1,1,0.75
SpotLightPos = 0.2032,1.8284,0.6998
LightSize = 0.05
LightFallOff = 0.01
LightGlowRad = 0
LightGlowExp = 1
BaseColor = 0.101961,0.0862745,0.0862745
OrbitStrength = 0.2
X = 0.454902,0.396078,0.329412,0.5
Y = 0.454902,0.345098,0.227451,0.5
Z = 0.533333,0.454902,0.388235,0.5
R = 0.772549,0.745098,0.713725,0.5
BackgroundColor = 1,1,1
GradientBackground = 0.2
CycleColors = false
Cycles = 1
FloorEnable = true
FloorMatte = false
FloorDebug = false
FloorNormal = 0,0,1
FloorHeight = 0
FloorColor = 1,1,1,0
HF_Fallof = 0.1
HF_Const = 0
HF_Intensity = 0
HF_Dir = 0,0,1
HF_Offset = 0
HF_Color = 1,1,1,1
HF_Scatter = 0
HF_Anisotropy = 0,0,0
HF_FogIter = 1
HF_CastShadow = false
mtime = 0
CloudScale = 1
CloudFlatness = 0
CloudTops = 0
CloudBase = -1
CloudDensity = 0
CloudRoughness = 1
CloudContrast = 1
CloudColor = 0.5,0.5,0.5
SunLightColor = 0.75,0.75,0.75
WindDir = 0,1,0
WindSpeed = 1
HCloudScale = 1
HCloudFlatness = 0.5
HCloudTops = 2
HCloudBase = 1
HCloudDensity = 0
HCloudRoughness = 1
HCloudContrast = 1
HCloudColor = 0.5,0.5,0.5
HCloudWindDir = 0,1,0
HCloudWindSpeed = 1
QTIterations = 16
QTThreshold = 24
QTC = 0.1,0.85,0.1,0
rotVector = 0,1,0
rotAngle = 0
Scale = 1
foldBoxStartIt = 256 Locked
foldBoxIts = 0 Locked
foldBoxLimit = 0,0,0
foldBoxValue = 2,2,2
foldBoxOffset = 0,0,0
foldSphereStartIt = 256 Locked
foldSphereIts = 3 Locked
foldSphereMinRad = 0.25
Background = //production/Public/Fractals/Fragmentarium/Camera Test/Frames/Test3/Test_00010.jpg
bgScale = 1,1
bgOffset = 1,1
DryPass = false
SpecularMax = 8
HardShadow = 1
ShadowSoft = 5
Up = -0.00154505,-0.0162073,0.999735
#endpreset

